	TITLE	'DSKT512 (512 Byte Disk Test) of 10-26-82'
;
;this program will read an entire disk and keep
;a running tab of errors. at the completion of the
;test, the type of errors found will be displayed.
;note: this program uses MODE 2 interrupts,
; (the system vector) for all disk interrupts.
;added time out for slim line drives (5-24-82)
;
	MACLIB	NEWMAC
	MACLIB	SKEW
;
IOBASE	EQU	80H		;i/o base address of cpz-48000
TRUE	EQU	0FFFFH
FALSE	EQU	NOT TRUE
FIRST	EQU	1		;first sector on disk
LAST	EQU	26		;last sector on disk
SKEWFSD	EQU	6		;skew factor for single density
SKEWFDD	EQU	1		;skew factor for double density
SECTDDB	EQU	16		;last sector for deblocked disk
SKFDEBL	EQU	4		;skew factor for deblocked
BELL	EQU	7		;bell code
BS	EQU	8		;back space code
BDOS	EQU	5
WBOOT	EQU	0
DMAP	EQU	IOBASE+70H	;dma base address
ADR2	EQU	DMAP+4		;dma address register port
WCT2	EQU	DMAP+5		;dma word count register port
CMND	EQU	DMAP+8		;dma command register port
FMASK	EQU	DMAP+10		;fixed mask register
MODE	EQU	DMAP+11		;dma mode register
MASK	EQU	DMAP+15		;dma mask register
;
CMD	EQU	IOBASE+10H	;disk base address
STATUS	EQU	CMD		;disk status port
TRK	EQU	CMD+1		;disk track register
SEC	EQU	CMD+2		;disk sector register
DAL	EQU	CMD+3		;disk data register
WAIT	EQU	IOBASE+53H	;disk wait port
DCONT	EQU	IOBASE+52H	;disk control port
;
RCMD	EQU	80H		;read with 15ms head settle time
STPRT	EQU	1		;0=3ms,1=6ms,2=10ms,3=15ms
HLAB	EQU	8		;0=no head load at beginning of seek
SEEK	EQU	10H+HLAB+STPRT	;seek command
RTYCNT	EQU	10		;10 retrys
INTRP	EQU	0D0H
RESTOR	EQU	STPRT		;restore command
SECS	EQU	LAST		;sectors/track
SSECS	EQU	26		;single density sectors/track
;
SIDE0	MACRO	DATA
	IF	NOT NUL DATA
	LDA	DATA		;;get storage value
	ENDIF
	ANI	10111000B
	OUT	DCONT		;;set to side 0
	ENDM
;
SIDE1	MACRO	DATA1
	IF	NOT NUL DATA1
	LDA	DATA1		;;get storage value
	ENDIF
	ORI	01000000B
	OUT	DCONT		;;change to side 1
	ENDM
;
FOLD	MACRO
	LOCAL	NOCONV
	CPI	'a'
	JC	NOCONV
	CPI	'z'+1
	JNC	NOCONV
	ANI	5FH
NOCONV:
	ENDM
;
	ORG	100H
;
START1:	LXI	SP,STACK	;set stack pointer.

	PRINT	<'I.C.M. Deblock Disk Test Program'>
	PRINT	<CR,LF,'Version 1.8 of 10-26-82'>
	PRINT	<CR,LF,CR,LF,'Test disks formatted with:'>
	PRINT	<CR,LF,'Track  0    = 26 128 byte sectors'>
	PRINT	<CR,LF,'Track  1-76 = double density'>
	PRINT	<CR,LF,'Deblocked 512 byte sectors'>
	PRINT	<CR,LF,LF,'Cntl-E Stops Testing.'>

	JMPR	STRTA
;
START:	CALL	RETNDR		;restore drive
STRTA:	LXI	H,00080H	;init. read operation
	SHLD	DMARED
	PRINT	<CR,LF,LF,'Starting Track or Cntl-C to stop '>
	INPUT	DCBUFF,6
	LDA	DCBUFF+1
	ORA	A		;was a carriage return pushed
	JRZ	ETRK		;do ending track
	LXI	H,DCBUFF+2	;else, get track requested to do
	LXI	B,0
	MOV	C,A
	DAD	B
	MVI	M,0FFH
	DECIN	DCBUFF+2
	STA	STRACK
;
ETRK:	PRINT	<CR,LF,'Ending Track or Cntl-C to stop '>
	INPUT	DCBUFF,6
	LDA	DCBUFF+1
	ORA	A		;carriage return pushed?
	JRZ	GETDRV		;use default if yes
	LXI	H,DCBUFF+2	;else, get ending track to do
	LXI	B,0
	MOV	C,A
	DAD	B
	MVI	M,0FFH
	DECIN	DCBUFF+2
	STA	ETRACK
GETDRV:	PRINT	<CR,LF,'Drive (A,B,C,D) or Cntl-C to stop? '>
	INPUT	DCBUFF,6
	LDA	DCBUFF+2
	FOLD
DRIVE:	ANI	3		;strip ascii bits
	DCR	A		;make it cpm drive number
	STA	THISDR		;save for ready message
	ADD	A
	ADD	A
	ADD	A
	ADD	A		;shift for latch select
	SIDE0			;select side 0 (default side).
	STA	CLATCH		;save drive code
	PRINT	<CR,LF,'Test a Double Sided Drive? (Y or N) '>
	INPUT	DCBUFF,6
	LDA	DCBUFF+2
	FOLD
	CPI	'N'		;no?
	JRZ	NOTSID
	PRINT	<CR,LF,'Side 0 or Side 1? (0 or 1) '>
	INPUT	DCBUFF,6
	LDA	DCBUFF+2
	CPI	'0'		;side 0?
	MVI	A,'0'		;show side we are doing
	STA	MSG4		;save for display
	JRZ	NOTSID
	INR	A		;make into side 1 message
	STA	MSG4
	SIDE1	CLATCH
	STA	CLATCH		;save the current drive side code.
NOTSID:	CALL	RDYDRV		;check for drive ready 1st
	CALL	RETNDR		;restore drive to home
	PRINT	<CR,LF>
DTESTF:	PRINT	<'Insert test disk and hit return (cr) '>
	INPUT	DCBUFF,6
	LDA	DCBUFF+1
	ORA	A		;must be a zero
	JRZ	DTESTE		;did not type cr, redo from start
	PRINT	<CR,'                                                  '>
	JMP	DTESTF		;reprint the question
;
RETNDR:	MVI	A,RESTOR
	OUT	CMD		;restore drive
	JMP	INTWAT		;wait for disk intrp
;
DTESTE:	PRINT	<CR,LF>
	CALL	RDYDRV		;check for drive ready
	LXI	H,SECNT		;point at error counters
	MVI	B,TK-SECNT	;get length of table
	XRA	A
CLRLP:	MOV	M,A
	INX	H
	DJNZ	CLRLP		;clear table
DOTEST:	LDA	STRACK
	STA	TK
	ORA	A		;check if going to trk zero
	CZ	SDTEST		;yes, change sector count.
	LXI	H,512		;dma length
	SHLD	DMARED		;set it
	LXI	H,SKEWDDB	;double density table
	SHLD	TABLE
	MOV	A,M		;get sector bbyte
	STA	SECT		;save it
	CALL	DENSET		;set up for double density
TLOOP:	XRA	A		;test loop
	STA	RCOUNT		;reset retry count
;
;seek retry entry point
;
SEEKE:	CALL	SEEKTK		;go seek to value in tk
	ANI	099H		;strip off junk bits
	JNZ	SERROR		;not 0 then log error
	CALL	CHECKRTY	;good ,go see if bad before
;
READLP:	XRA	A		;get a zero
	STA	RCOUNT		;reset retry count
;
;retry entry point for readsec
;
READE:	CHARSTAT		;check for a key pushed
	JRZ	READE1		;skip if no char.
	CHARIN			;get it
	CPI	'E'-40H		;cntl E?
	JZ	FORGET		;exit if yes
READE1:	CALL	READSC		;go read sector pointed to by sect
	ORA	A		;0 = good
	JNZ	RERROR		;if bad go log and retry
;
;entry when max retrys is reached
;
SKIPIT:	CALL	CHECKRTY	;good ,go see if bad before
	LHLD	TABLE		;get skew table address
	INX	H
	SHLD	TABLE
	MOV	A,M		;get sector number
	ORA	A		;is it end of table?
	JRZ	NEWTK
SAMETK:	STA	SECT		;update sector pointer
	JMPR	READLP		;go do more
;
;increment track (tk)
;
NEWTK:	CALL	DENSET		;change to double density
	LDA	TK		;get track number
	INR	A
	MOV	B,A
	LDA	ETRACK		;get last track to test
	CMP	B		;check it aginst new track
	JM	TOTAL		;if new tk > last tk then done
	MOV	A,B		;get new track
	STA	TK		;update tk pointer
	LXI	H,512		;read 512 bytes
	SHLD	DMARED
	LXI	H,SKEWDDB	;point to deblock sector table
NEWTKD:	SHLD	TABLE
	MOV	A,M		;get first sector
	STA	SECT
	JMP	TLOOP
;
SDTEST:	LDA	CLATCH		;are we doing double sided?
	ANI	40H		;check bit 6 for set.
	CPI	40H		;is it?
	RZ			;if so, trk 0 - side 1 = doub den.
	POP	H		;else, trk 0 side 0 = sing. den.
	LXI	H,SKEWSD	;point to single density table
	SHLD	TABLE
	MOV	A,M		;get sector number
	STA	SECT		;save it
	LDA	CLATCH		;get current density and drive
	ANI	0F7H		;make single density
	STA	CLATCH		;save it
	OUT	DCONT		;change density
	JMP	TLOOP		;read single density
;
DENSET:	LDA	CLATCH
	ORI	8		;double density enable
	STA	CLATCH
	OUT	DCONT
	RET
;
SERROR:	MOV	B,A		;enter here from seek error
	INXM	SECNT		;inc seek error counter
	LDA	RCOUNT		;get retry counter
	INR	A
	CPI	RTYCNT+1	;see if max retrys
	JZ	FORGET
	CPI	1		;see if first retry
	JNZ	SEEKE		;not first then just retry
	CALL	STATS		;first error so print prameters
	JMP	SEEKE		;now do retry
;
;enter for read errors
;
RERROR:	MOV	B,A		;save error
	ANI	4		;lost data?
	JRZ	E1		;0=no
	INXM	LDCNT
E1:	MOV	A,B		;get error status
	ANI	8		;crc error?
	JRZ	E2
	INXM	CRCNT
E2:	MOV	A,B
	ANI	0E3H		;check other bits
	JRZ	E4
	INXM	OTHER
E4:	MOV	A,B
	ANI	10H
	JRZ	E5
	INXM	IDCNT
E5:	LDA	RCOUNT
	INR	A
	CPI	RTYCNT+1
	STA	RCOUNT
	JZ	SKIPIT		;past max retrys then skip sector
	CPI	1
	JNZ	READE		;not first error then just retry
	CALL	STATS		;first error, print it
	JMP	READE		;now do retry
;
CHECKRTY:
	LDA	RCOUNT		;get retry count
	ORA	A		;see if its zero
	RZ			;if 0 then dont do anything
	LXI	H,RTYMSG1	;point at middle of retry message
	CALL	BIND1		;convert to decimal and put in message
	LXI	D,RTYMSG	;point at retry message
	MVI	C,9
	JMP	BDOS		;print it
;
FORGET:	LXI	D,FMSG		;message
	MVI	C,9
	CALL	BDOS
;
TOTAL:	PRINT	<CR,LF,LF,'Seek Errors  '>
	LHLD	SECNT
	DECOUT
	PRINT	<CR,LF,'Lost Data    '>
	LHLD	LDCNT
	DECOUT
	PRINT	<CR,LF,'CRC Errors   '>
	LHLD	CRCNT
	DECOUT
	PRINT	<CR,LF,'ID Errors    '>
	LHLD	IDCNT
	DECOUT
	PRINT	<CR,LF,'Other Errors '>
	LHLD	OTHER
	DECOUT
	JMP	START
;
READSC:	MVI	A,0D0H		;force intrp command
	OUT	CMD		;send to floppy
	LHLD	DMARED		;get operation
	DCX	H		;count = count-1
	MOV	A,L		;read count byte
	OUT	WCT2		;tell dma controller
	MOV	A,H		;get read command
	OUT	WCT2		;  and tell dma controller
	LXI	H,RDBUFF	;point to buffer
	MOV	A,L		;get low memory address byte
	OUT	ADR2		;  and send it to dma controller
	MOV	A,H		;get high memory address byte
	OUT	ADR2		;  and send it to dma controller
	LDA	SECT
	OUT	SEC		;tell 1793 what sector to read
	MVI	A,46H
	OUT	MODE
	MVI	A,70H		;dma read command
	OUT	CMND
	MVI	A,02H		;clear mask for chan 2 dma xfer
	OUT	FMASK
	LDA	SEKFLG		;did we seek new track?
	ORA	A		;0 = no, 1 = yes
	MVI	A,0		;reset it just in case we did
	STA	SEKFLG
	JRZ	NONSEK		;dont enable 15ms delay if not
	MVI	B,RCMD OR 4	;enable 15ms delay on head settle
	JMPR	DIDSEK
;
NONSEK:	MVI	B,RCMD
DIDSEK:	CALL	HDLD
	ORA	B
	OUT	CMD
INTWAT:	HLT			;stop processor
	JMPR	INTWAT		;loop until disk intrp
;
HDLD:	IN	CMD
	ANI	20H
	MVI	A,4
	RZ
	MVI	A,0
	RET
;
SEEKTK:	LDA	TK
	OUT	DAL 		;tell 1793
	MVI	A,SEEK
	OUT	CMD
	CALL	INTWAT		;wait for disk intrp
	MVI	A,1		;set track seek flag
	STA	SEKFLG		;we did a seek to next track
	IN	STATUS		;get disk status
	RET
;
STATS:	LDA	F1793		;get status bit msg flag byte
	ORA	A		;is it zero?
	JRNZ	DERROR		;no, jmp past 1793 flag msg
	CMA			;yes, print 1793 flag bytes once
	STA	F1793		;and make flag byte non-zero.
	PUSH	B
	PRINT	<CR,LF,CR,LF,'1793  RDY  -  RDT RNF CRC LSD DRQ BSY'>
	POP	B
DERROR:	MOV	A,B
	LXI	H,MSG1
	PUSH	D
	CALL	BINCON		;convert to hex
	POP	D
	LXI	H,MSG2
	IN	TRK
	CALL	BIND1
	LXI	H,MSG3
	IN	SEC
	CALL	BIND1
	MVI	C,9
	LXI	D,MSG0
	JMP	BDOS
;
BINCON:	MVI	B,8		;loop count
	LXI	D,4		;byte count
BLOOP:	RLC			;check for binary 1
	JRC	BONE		;jump if it is
	MVI	M,'0'		;no, show ascii zero
BUMP:	DAD	D		;bump pointer
	DJNZ	BLOOP
	RET
;
BONE:	MVI	M,'1'		;bit is a binary 1
	JMPR	BUMP
;
CONVT:	MOV	D,A		;save the value
	RAR
	RAR
	RAR
	RAR
	CALL	BIN1		;convert to ascii
	MOV	A,D
BIN1:	ANI	0FH
	ADI	'0'
	CPI	3AH
	JM	DONE
	ADI	7
DONE:	STAX	B
	INX	B
	RET
;
MSG0:	DB	0AH,0DH,'Status '
MSG1:	DB	'                               '
	DB	' Side '
MSG4:	DB	'0'
	DB	' Track '
MSG2:	DB	'00'
	DB	' Sector '
MSG3:	DB	'00'
	DB	'$'
RTYMSG:	DB	' Retrys '
RTYMSG1:DB	'00'
	DB	'$'
FMSG:	DB	0DH,0AH
	DB	0DH,0AH,'F O R G E T  T H I S ! ! !',0DH,0AH,'$'
;
BIND1:	MVI	B,10
	CALL	BID1
	ADI	'0'
	MOV	M,A
	RET
;
BID1:	MVI	M,'0'-1
	INR	M
	SUB	B
	JRNC	BID1+2
	ADD	B
	INX	H
	RET
;
;check to see if selected drive is ready.
;this routine is used to check slim line drives also.
;
RDYDRV:	IN	STATUS		;check status
	RLC			;is drive ready?
	RNC			;return if ready
;
;if drive not ready (probably slim line drive), seek to
;same track to start motor.
;
	IN	TRK		;get present track number
	OUT	DAL		;place into data reg
	MVI	A,00011001B	;seek to same track to start drive
	OUT	CMD		;send it
	CALL	INTWAT		;wait for intrp
;
	MVI	B,50		;set for 50 * (16 * 1 ms) time delay
DELAY:	MVI	A,16		;16 ms delay constant
DELY1:	MVI	C,249		;1 ms constant (4 MHz cpu speed)
DLY1:	DCR	C		;decrease count
	JRNZ	DLY1		;loop till = 0
	DCR	A		;decrease count factor
	JRNZ	DLY1		;loop for 16 ms
	DCR	B		;800 ms loop
	JRZ	NOTRDY		;print which drive is not ready
	IN	STATUS
	RLC
	RNC			;return if ready
	JMPR	DELAY		;check for 800 ms before giving up.
;
;drive has been found not ready at all.
;
NOTRDY:	LDA	THISDR		;get requested disk number
	ADI	'A'		;convert to CP/M drive number
	STA	NDSKR		;save for message
	LXI	D,NTRDYM	;point to not ready message
	MVI	C,9		;print function
	CALL	BDOS		;print it
;
;hang waiting for disk to come ready.
;
DRVRDY:	IN	STATUS		;get disk status
	RLC
	JRC	DRVRDY		;loop till ready
	RET
;
NTRDYM:	DB	CR,LF,BEL,'```E R R O R```  Drive '
NDSKR:	DB	' : is  N O T  Ready.$'
;
SEKFLG:	DB	1		;flag for read delay after 1st seek
STRACK:	DB	0		;starting track
ETRACK:	DB	76		;ending track
RCOUNT:	DB	0		;retry counter
SECNT:	DW	0		;seek error counter
LDCNT:	DW	0		;lost data counter
CRCNT:	DW	0		;crc error counter
IDCNT:	DW	0		;id not found error
OTHER:	DW	0		;other errors
F1793:	DB	0		;0=print 1793 error bits message once.
TK:	DB	1		;start at track 1+1
SECT:	DB	1		;flopy sector number
ESECT:	DB	SECS+1		;max sectors per disk
TABLE:	DW	0		;skew table address storage
DMARED:	DW	80H		;dma read operation storage,default to 128 byte
SKEWSD:	SKEW	FIRST,SSECS,SKEWFSD
SKEWDDB:SKEW	FIRST,SECTDDB,SKFDEBL
	DS	64		;stack area
STACK:	DS	1
THISDR:	DS	1		;current physical drive
CLATCH:	DS	1		;current drive code
DCBUFF:	DS	9
RDBUFF:	DS	1		;read buffer
	END	START1
