	TITLE	'512 Byte Sysgen Program of 10-26-82'
;
;program to read or write a 512 x 16
;deblocked disk. this program is just like sysgen.
;
;track 0 = single density 128 bytes x 26 sectors.
;track 1 = double density 512 bytes x 16 sectors.
;
;all of track 1 is read or written with this program.
;if running double sided, track 1 now becomes track 0
;on side 1. it is double density, with 16 sectors of
;512 bytes.
;
;this program uses the system interrupt for the disk.
;assemble using zasm.com (z-80 code)
;
;system equates
;
IOBASE	EQU	80H		;i/o base address of cpz-48000
;
DISK	EQU	IOBASE+10H	;disk port base address.
STPRAT	EQU	1		;0=3ms,1=6ms,2=10ms,3=15ms
ADR2	EQU	IOBASE+74H	;dma address port.
WCT2	EQU	IOBASE+75H	;dma word count port.
CMND	EQU	IOBASE+78H	;dma command port.
FMASK	EQU	IOBASE+7AH	;dma fixed mask register
MODE	EQU	IOBASE+7BH	;dma mode register
CLRBP	EQU	IOBASE+7CH	;dma clear byte f/f
DCOM	EQU	DISK		;command port.
DSTAT	EQU	DISK		;status port.
FTRK	EQU	DISK+1		;floppy track register
SECT	EQU	DISK+2		;sector port.
DDATA	EQU	DISK+3		;floppy data reg
DCONT	EQU	IOBASE+52H	;control port.
SYSTEM	EQU	0900H		;start of cpm image
SDSPT	EQU	26		;26 sectors / track.
DDSPT	EQU	16		;16 sectors / track.
NSECTS	EQU	SDSPT + DDSPT	;total system sectors.
RTCNT	EQU	10		;number of retrys.
B128	EQU	127		;128 byte count
B512	EQU	511		;512 byte count
CR	EQU	0DH
LF	EQU	0AH
BEL	EQU	7		;bell code
IDBUF	EQU	97EH		;id byte location in memory
BOOT	EQU	0
BDOS	EQU	5
PRINTB	EQU	9		;print buffer function
RCB	EQU	10		;read console buffer <func 10>
;
	ORG	100H		;run area
;
BEGIN:	JMP	INIT		;jump around storage
;
MSG1:	DB	CR,LF,'I.C.M. 512 x 16 Sysgen program ver 1.3 of 10-26-82.'
MSG1A:	DB	CR,LF,'Source drive or <cr to skip> $'
MSG3:	DB	CR,LF,'Source drive = $'
MSG2:	DB	CR,LF,'Destination drive or <cr to reboot> $'
MSG4:	DB	CR,LF,'Destination drive = $'
READY:	DB	' , type return when ready$'
WRITER:	DB	CR,LF,BEL,'Disk write error$'
RRERR:	DB	CR,LF,BEL,'Disk read error$'
ERROR:	DB	CR,LF,BEL,'Invalid drive select$'
NTRDYM:	DB	CR,LF,BEL,'\\\ERROR\\\ Drive '
NDSKR:	DB	' : is not READY.$'
MSG5:	DB	CR,LF,'Reading a Double Sided disk (Y or N) $'
MSG6:	DB	CR,LF,'Writing a Double Sided disk (Y or N) $'
NMERR:	DB	CR,LF,BEL,'Mis-match on drive ID byte.$'
KEYBUF:	DB	4,0,0,0,0,0
;
;subroutines
;
;input routine - get a character from keyboard
;
;enter:	none
;exit:	reg a = char from keyboard
;	stored at seld
;registers used: d,e h,l c
;
INPUT:	MVI	A,4		;reset buffer length
	LXI	H,0
	SHLD	KEYBUF+1	;clear buffer
	STA	KEYBUF
	LXI	D,KEYBUF	;point to buffer
	MVI	C,RCB		;read buffer function
	CALL	BDOS		;get character
	LDA	KEYBUF+2	;get the first char only
	CPI	'a'		;check for lower case
	RC
	CPI	'z'+1
	RNC
	ANI	5FH		;make into upper case
	RET
;
;print buffer routine
;
;enter:	d,e -> message to print
;exit:	none
;regs:	d,e c
;
PRINT:	MVI	C,PRINTB	;print buffer function
	JMP	BDOS		;use bdos to print
;
;select drive routine
;
DSELDSK:
	STA	THISDR		;current physical drive select
	ADD	A		;shift
	ADD	A		; left
	ADD	A		;  four
	ADD	A		;   times
	OUT	DCONT		;send the select code
	STA	CLATCH		;save it
	RET
;
;home selected drive
;
DHOME:	MVI	A,STPRAT	;restore at step rate
	OUT	DCOM
INTWAT:	HLT			;wait for intrp
	JMPR	INTWAT
;
;main line starts here
;
INIT:	LXI	SP,STACK	;set local stack
	LXI	D,MSG1		;show opening message
	OUT	CLRBP		;clear byte pointer f/f
GDRV:	CALL	PRINT
	CALL	INPUT		;get source drive
	ORA	A		;= carriage return?
	JRZ	DESTDRV		;yes, memory already set
	STA	READY		;show drive selected
	ANI	7		;strip ascii
	DCR	A		;make cpm drive number
	CPI	4		;check drive limits
	JRC	OK
	LXI	D,ERROR		;point to error message
	CALL	PRINT
	LXI	D,MSG1A
	JMPR	GDRV		;try again
;
OK:	STA	SELD		;modified for cpm select
	LXI	D,MSG5		;reading double sided?
	CALL	PRINT
	CALL	INPUT		;get response
	CPI	'N'		;no?
	MVI	A,0		;0 = doing single sided disk
	JRZ	OK2		;yes, just doing single sided
	MVI	A,1		;doing double sided, set a flag
OK2:	STA	SIDED		;save the flag
	LXI	D,MSG3
	CALL	PRINT
	LXI	D,READY		;print ready message
	CALL	PRINT
OK1:	CALL	INPUT		;get cr
	ORA	A		;did we get a cr?
	JRNZ	OK1		;loop till a cr is typed
	LDA	SELD		;get drive to read
	CALL	DSELDSK
	CALL	RDYDRV		;wait till drive is ready
	CALL	DHOME		;home the drive
	CALL	READDSK		;read the selected drive
;
DESTDRV:LXI	D,MSG2		;get destination drive to do
DDRV:	CALL	PRINT
	CALL	INPUT		;get source drive
	ORA	A		;= 0?
	JZ	BOOT		;all done
	STA	READY		;show drive selected
	ANI	7		;strip ascii
	DCR	A		;make cpm drive number
	CPI	4		;check drive limits
	JRC	DOK
	LXI	D,ERROR		;point to error message
	CALL	PRINT
	LXI	D,MSG1A
	JMPR	DDRV		;try again
;
DOK:	STA	SELD		;modified for cpm select
	LXI	D,MSG6		;writing double sided drive
	CALL	PRINT
	CALL	INPUT		;get response
	CPI	'N'		;is it no?
	MVI	A,0		;set flag for no in case
	MVI	B,0DEH		;talking to single sided?
	JRZ	DOK2
	MVI	A,1		;yes, set = doub sided wanted
	MVI	B,0DCH		;must be using double sided
DOK2:	STA	SIDED		;save the flag code
	LDA	IDBUF		;get the id byte in memory
	CMP	B		;are they the same?
	JRNZ	NOMTCH		;jump if they dont match
	LXI	D,MSG4		;point to message
	CALL	PRINT
	LXI	D,READY		;ready message
	CALL	PRINT
DOK1:	CALL	INPUT		;get a cr
	ORA	A		;must be zero
	JRNZ	DOK1		;loop till cr
	LDA	SELD		;get the selected drive
	CALL	DSELDSK		;select it
	CALL	RDYDRV		;wait till ready
	CALL	DHOME		;home the drive
	CALL	DSKWRT		;write the operating system
	JMP	DESTDRV
;
;check to see if selected drive is ready.
;this routine is used to check slim line drives also.
;
RDYDRV:	IN	DSTAT		;check status
	RLC			;is drive ready?
	RNC			;return if ready
;
;if drive not ready (probably slim line drive), seek to
;same track to start motor.
;
	IN	FTRK		;get present track number
	OUT	DDATA		;place into data reg
	MVI	A,00011001B	;seek to same track to start drive
	OUT	DCOM		;send it
	CALL	INTWAT		;wait for intrp
;
	MVI	B,50		;set for 50 * (16 * 1 ms) time delay
DELAY:	MVI	A,16		;16 ms delay constant
	MVI	C,249		;1 ms constant (4 MHz cpu speed)
DLY1:	DCR	C		;decrease count
	JRNZ	DLY1		;loop till = 0
	DCR	A		;decrease count factor
	JRNZ	DLY1		;loop for 16 ms
	DCR	B		;800 ms loop
	JRZ	NOTRDY		;print which drive is not ready
	IN	DSTAT
	RLC
	RNC			;return if ready
	JMPR	DELAY		;check for 800 ms before giving up.
;
;drive has been found not ready at all.
;
NOTRDY:	LDA	THISDR		;get requested disk number
	ADI	'A'		;convert to CP/M drive number
	STA	NDSKR		;save for message
	LXI	D,NTRDYM	;point to not ready message
	CALL	PRINT		;print it
;
;hang waiting for disk to come ready.
;
DRVRDY:	IN	DSTAT
	RLC
	JRC	DRVRDY		;loop till ready
	RET
;
NOMTCH:	LXI	D,NMERR		;point to mismatch error
	CALL	PRINT		;print it
	JMP	DESTDRV		;retry
;
READDSK:LXI	H,B128		;do 128 bytes
	SHLD	WRITOP
	MVI	A,46H		;single xfer,write xfer,chan2
	STA	DMODE
	MVI	A,50H		;dreq low, rotating priority
	STA	DCMND
	MVI	A,SDSPT+1	;number of single density sects
	STA	RSECTSZ+1
	MVI	E,RTCNT		;get retry count.
RBLOOP:	LXI	H,SYSTEM	;cp/m starts here.
	SHLD	SYSVAL		;save it
	MVI	D,NSECTS	;number of sectors to write.
RNTRK:	MVI	C,1		;sector number.
	MVI	B,080H		;for head load.
RNSEC:	CALL	RRED		;read first sector.
	DCR	D		;if done,
	RZ
	INR	C		;increment sector count.
	MOV	A,C		;done with
RSECTSZ:CPI	SDSPT+1		;this track?
	JRC	RNSEC		;if not, read next sector.
	MVI	A,DDSPT+1	;number of doub den sectors
	STA	RSECTSZ+1	;modify the code.
	LXI	H,B512		;do 512 bytes
	SHLD	WRITOP		;do 512 byte write
	CALL	SETDEN		;set up for double density
	JMPR	RNTRK
;
SETDEN:	LDA	CLATCH		;get current code
	ORI	8		;set double density code
	STA	CLATCH		;save it
	OUT	DCONT		;set latch for d.density
	LDA	SIDED		;doing doub sided?
	ORA	A		;= to 0?
	JRNZ	RSIDED		;not =, doing double sided
	MVI	A,5BH		;step command.
	OUT	DCOM		;issue it.
	JMP	INTWAT
;
RSIDED:	LDA	CLATCH		;get current drive code
	ORI	40H		;switch side
	OUT	DCONT		;switch side now
	RET
;
RRED:	CALL	DMARW		;use common routine
	SHLD	SYSVAL		;save next address
	IN	DSTAT		;read status.
	ANI	09DH		;look at error bits.
	RZ			;ok if zero.
	DCR	E		;decrement retry count.
	JRNZ	RBLOOP		;try again if not zero.
	LXI	D,RRERR		;retry exausted
	CALL	PRINT		;print the error message
	JMP	BOOT		; and return to cpm.
;
DSKWRT:	LXI	H,B128		;write 128 bytes
	SHLD	WRITOP
	MVI	A,4AH		;single xfer, read xfer,chan2
	STA	DMODE
	MVI	A,70H		;dreq low, extended write, rot pri.
	STA	DCMND
	MVI	A,SDSPT+1	;number of single density sects
	STA	SECTSZ+1
	MVI	E,RTCNT		;get retry count.
BLOOP:	LXI	H,SYSTEM	;cp/m starts here.
	SHLD	SYSVAL		;save it
	MVI	D,NSECTS	;number of sectors to write.
WNTRK:	MVI	C,1		;sector number.
	MVI	B,0A0H		;for head load.
WNSEC:	CALL	WRIT		;write first sector.
	DCR	D		;if done,
	RZ
	INR	C		;increment sector count.
	MOV	A,C		;done with
SECTSZ:	CPI	SDSPT+1		;this track?
	JRC	WNSEC		;if not,write next sector.
	MVI	A,DDSPT+1	;number of doub den sectors
	STA	SECTSZ+1	;modify the code.
	LXI	H,B512		;write 512 bytes
	SHLD	WRITOP		;do 512 byte write
	CALL	SETDEN
	JMPR	WNTRK		;write next track.
;
WRIT:	CALL	DMARW		;use common routine
	SHLD	SYSVAL		;save next address
	IN	DSTAT		;read status.
	ANI	0FDH		;look at error bits.
	RZ			;ok if zero.
	DCR	E		;decrement retry count.
	JRNZ	BLOOP		;try again if not zero.
	LXI	D,WRITER
	CALL	PRINT
	JMP	BOOT
;
DMARW:	MVI	A,0D0H		;force intrp command
	OUT	DCOM		;clear floppy chip
	PUSH	D		;save d,e
	LDED	WRITOP		;get write operation
	LHLD	SYSVAL		;get memory pointer
	MOV	A,E		;count byte
	OUT	WCT2
	MOV	A,D		;write command
	OUT	WCT2
	INX	D		;adjust dumb counter
	MOV	A,L		;get low address byte
	OUT	ADR2
	MOV	A,H		;high address byte
	OUT	ADR2
	DAD	D		;h,l = (h,l + d,e)
	POP	D		;restore d,e
	LDA	DMODE		;get mode to do
	OUT	MODE
	LDA	DCMND
	OUT	CMND
	MVI	A,2		;request chan2
	OUT	FMASK
	MOV	A,C		;sector in a.
	OUT	SECT		;set sector register.
	CALL	HDLD		;check for head loaded
	ORA	B		;get read/write command
	OUT	DCOM		;issue command.
	JMP	INTWAT
;
;check head load bit
;
HDLD:	IN	DSTAT		;get disk status
	ANI	20H		;mask head load bit
	MVI	A,4		;get a 4 for load head
	RZ			;reload the head
	MVI	A,0		;else, head is load already
	RET
;
THISDR:	DS	1		;current physical drv select
DMODE:	DS	1		;dma mode byte save
DCMND:	DS	1		;dma command byte save
WRITOP:	DS	2		;dma write operation storage
SYSVAL:	DS	2		;system pointer storage
CLATCH:	DS	1		;current drive select code
SIDED:	DS	1		;doub sided flag byte
SELD:	DS	1		;current drive to select
	DS	30		;stack area
STACK:	DS	1
;
	END	BEGIN

